@@ -38,6 +38,9 @@ from bzrlib.tests import blackbox
 class TestInfoXml(blackbox.ExternalBase):
 
     def test_should_normalize_non_ascii_url(self):
+        # we disable isolation because the error we want to catch is triggered
+        # outside of the jail
+        self.disable_directory_isolation()
         root = '/C:' if sys.platform == 'win32' else ''
         url = u'file://%s/Maçã/does/not/exist' % root
         out, err = self.run_bzr(['xmlinfo', url], retcode=3)
@@ -46,22 +49,29 @@ class TestInfoXml(blackbox.ExternalBase):
         self.assertEqual(
             '<?xml version="1.0" encoding="%s"?>'
             '<error><class>NotBranchError</class><dict><key>path</key>'
-            '<value>%s/</value></dict><message>Not a branch: "%s/".</message>'
+            '<value>%s/</value><key>extra</key><value></value>'
+            '<key>detail</key><value></value></dict>'
+            '<message>Not a branch: "%s/".</message>'
             '</error>' % (osutils.get_user_encoding(), nurl, nurl), err)
 
-
     def test_info_non_existing(self):
+        # we disable isolation because the error we want to catch is triggered
+        # outside of the jail
+        self.disable_directory_isolation()
         if sys.platform == "win32":
             location = "C:/i/do/not/exist/"
         else:
             location = "/i/do/not/exist/"
         out, err = self.run_bzr('xmlinfo '+location, retcode=3)
         self.assertEqual(out, '')
-        self.assertEqual('<?xml version="1.0" encoding="%s"?><error>'
-                '<class>NotBranchError</class><dict><key>path</key><value>'
-                '%s</value></dict><message>Not a branch: "%s".</message>'
-                '</error>' % (osutils.get_user_encoding(),
-                              location, location), err)
+        self.assertEqual(
+            '<?xml version="1.0" encoding="%s"?><error>'
+            '<class>NotBranchError</class><dict><key>path</key><value>'
+            '%s</value><key>extra</key><value></value>'
+            '<key>detail</key><value></value></dict>'
+            '<message>Not a branch: "%s".</message>'
+            '</error>' % (osutils.get_user_encoding(),
+                          location, location), err)
 
     def test_info_standalone(self):
         transport = self.get_transport()
@@ -228,8 +238,8 @@ class TestInfoXml(blackbox.ExternalBase):
 <added>0</added>
 <removed>0</removed>
 <renamed>0</renamed>
-<unknown>1</unknown>
-<ignored>0</ignored>
+<unknown>0</unknown>
+<ignored>1</ignored>
 <versioned_subdirectories>0</versioned_subdirectories>
 </working_tree_stats>
 <branch_history>
@@ -311,7 +321,6 @@ class TestInfoXml(blackbox.ExternalBase):
 <layout>Lightweight checkout</layout>
 <formats>
 <format>2a</format>
-<format>development-subtree</format>
 </formats>
 <location>
 <light_checkout_root>lightcheckout</light_checkout_root><checkout_of_branch>standalone</checkout_of_branch></location>
@@ -427,8 +436,8 @@ class TestInfoXml(blackbox.ExternalBase):
 <added>0</added>
 <removed>0</removed>
 <renamed>0</renamed>
-<unknown>1</unknown>
-<ignored>0</ignored>
+<unknown>0</unknown>
+<ignored>1</ignored>
 <versioned_subdirectories>0</versioned_subdirectories>
 </working_tree_stats>
 <branch_history>
@@ -505,7 +514,6 @@ class TestInfoXml(blackbox.ExternalBase):
 <layout>Lightweight checkout</layout>
 <formats>
 <format>2a</format>
-<format>development-subtree</format>
 </formats>
 <location>
 <light_checkout_root>lightcheckout</light_checkout_root><checkout_of_branch>standalone</checkout_of_branch></location>
@@ -672,7 +680,6 @@ class TestInfoXml(blackbox.ExternalBase):
 <layout>Lightweight checkout</layout>
 <formats>
 <format>2a</format>
-<format>development-subtree</format>
 </formats>
 <location>
 <light_checkout_root>tree/lightcheckout</light_checkout_root><checkout_of_branch>repo/branch</checkout_of_branch><shared_repository>repo</shared_repository></location>
@@ -816,7 +823,6 @@ class TestInfoXml(blackbox.ExternalBase):
 <layout>Lightweight checkout</layout>
 <formats>
 <format>2a</format>
-<format>development-subtree</format>
 </formats>
 <location>
 <light_checkout_root>tree/lightcheckout</light_checkout_root><checkout_of_branch>repo/branch</checkout_of_branch><shared_repository>repo</shared_repository></location>
@@ -1285,10 +1291,9 @@ class TestInfoXml(blackbox.ExternalBase):
             except errors.PathNotChild:
                 return path
 
-        if tree_locked and sys.platform == 'win32':
-            # We expect this to fail because of locking errors. (A write-locked
-            # file cannot be read-locked in the same process).
-            # This should be removed when the locking errors are fixed.
+        if tree_locked:
+            # We expect this to fail because of locking errors, dirstate
+            # can't be read locked while a write lock is open.
             self.run_bzr_error([], 'xmlinfo ' + command_string)
             return
         out, err = self.run_bzr('xmlinfo %s' % command_string)
@@ -1298,8 +1303,7 @@ class TestInfoXml(blackbox.ExternalBase):
             (False, True): 'Lightweight checkout',
             (False, False): 'Checkout',
             }[(shared_repo is not None, light_checkout)]
-        format = {True: '<format>2a</format>\n' +
-                        '<format>development-subtree</format>\n',
+        format = {True: '<format>2a</format>\n',
                   False: '<format>unnamed</format>'}[light_checkout]
         if repo_locked:
             repo_locked = lco_tree.branch.repository.get_physical_lock_status()
@@ -1412,7 +1416,8 @@ class TestInfoXml(blackbox.ExternalBase):
         # Do a light checkout of the heavy one
         transport.mkdir('tree/lightcheckout')
         lco_dir = bzrlib.bzrdir.BzrDirMetaFormat1().initialize('tree/lightcheckout')
-        bzrlib.branch.BranchReferenceFormat().initialize(lco_dir, co_branch)
+        bzrlib.branch.BranchReferenceFormat().initialize(lco_dir,
+                                                         target_branch=co_branch)
         lco_dir.create_workingtree()
         lco_tree = lco_dir.open_workingtree()
 
@@ -1509,6 +1514,11 @@ class TestInfoXml(blackbox.ExternalBase):
     def test_info_locking_oslocks(self):
         if sys.platform == "win32":
             raise tests.TestSkipped("don't use oslocks on win32 in unix manner")
+        # This test tests old (all-in-one, OS lock using) behaviour which
+        # simply cannot work on windows (and is indeed why we changed our
+        # design. As such, don't try to remove the thisFailsStrictLockCheck
+        # call here.
+        self.thisFailsStrictLockCheck()
 
         tree = self.make_branch_and_tree('branch',
                                          format=bzrlib.bzrdir.BzrDirFormat6())
@@ -1601,3 +1611,27 @@ class TestInfoXml(blackbox.ExternalBase):
         self.assertEqualDiff(expected_xml, out)
         self.assertEqual('', err)
         tree.unlock()
+
+    def test_info_stacked(self):
+        # We have a mainline
+        trunk_tree = self.make_branch_and_tree('mainline', format='1.6')
+        trunk_tree.commit('mainline')
+        # and a branch from it which is stacked
+        new_dir = trunk_tree.bzrdir.sprout('newbranch', stacked=True)
+        out, err = self.run_bzr('xmlinfo  newbranch')
+        expected_xml = '''<?xml version="1.0"?>
+<info>
+<layout>Standalone tree</layout>
+<formats>
+<format>1.6</format>
+</formats>
+<location><branch_root>newbranch</branch_root></location>
+<related_branches>
+<parent_branch>mainline</parent_branch>
+<stacked_on>mainline</stacked_on>
+</related_branches>
+</info>'''
+        expected_xml = ''.join(expected_xml.split('\n'))+'\n'
+        self.assertEqual(expected_xml, out)
+        self.assertEqual("", err)
+
